// 
//  MAC_MP3: A Low Energy Implementation of an Audio Decoder
//
//  Copyright (C) 2007 McMaster University
// 
//==============================================================================
// 
// This file is part of MAC_MP3
// 
// MAC_MP3 is distributed in the hope that it will be useful for further research,
// but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. MAC_MP3 is free; you can redistribute it
// and/or modify it provided that proper reference is provided to the authors. See
// the documents included in the "doc" folder for further details.
//
//==============================================================================


`timescale 1 ns/ 100 ps

`include "defines.v"

`define EOF 32'hFFFF_FFFF
`define NULL 0
`define MAX_LINE_LENGTH 1000

`define NUM_MAC_SAMPLE 'd576
`define NUM_PCM_SAMPLE 'd576
`define NUM_OF_FRAME 'd400
`define NUM_OF_GRANULE 'd2
`define NUM_OF_CHANNEL 'd2

`define NUM_OF_BLOCK 200

`define SKIP_FRAME 0

// For debug data
//`define DEBUG_HUFF
//`define DEBUG_REQ
//`define DEBUG_ANTI_OUT
//`define DEBUG_ICG_NORMAL_CH0
//`define DEBUG_ICG_NORMAL_CH1
//`define DEBUG_ICG_SHORT_CH0
//`define DEBUG_ICG_SHORT_CH1
//`define DEBUG_IPG_NORMAL_CH0
//`define DEBUG_IPG_NORMAL_CH1
//`define DEBUG_FREQ_INV
//`define DEBUG_MDCT_CH0
//`define DEBUG_MDCT_CH1
//`define DEBUG_POLY_WINDOW_CH0
//`define DEBUG_POLY_WINDOW_CH1

// For comparison
//`define DEBUG_COMP_REQ
`define DEBUG_COMP_ANTI_IN
`define DEBUG_COMP_ICG_B4_WIN_CH0
`define DEBUG_COMP_ICG_B4_WIN_CH1
`define DEBUG_COMP_ICG_CH0
`define DEBUG_COMP_ICG_CH1
//`define DEBUG_COMP_IPG_B4_WIN_CH0
//`define DEBUG_COMP_IPG_B4_WIN_CH1
//`define DEBUG_COMP_IPG_CH0
//`define DEBUG_COMP_IPG_CH1
`define DEBUG_COMP_MDCT_CH0
`define DEBUG_COMP_MDCT_CH1
`define DEBUG_COMP_PCM_CH0
`define DEBUG_COMP_PCM_CH1

module mp3_top_tb_mp3_v_tf();

// DATE:     10:46:06 07/25/2005 
// MODULE:   mp3_top
// DESIGN:   mp3_top
// FILENAME: tb_mp3.v
// PROJECT:  mp3_top
// VERSION:  


// Inputs
    reg HUFF_CLOCK_I;
    reg MAC_CLOCK_I;
    reg RESETN_I;
    
    reg NIOS_WRITE_DATA_CLK_I;
    reg [`ADDRESS_WIDTH-1:0]	NIOS_WRITE_DATA_ADDRESS_I;
    reg [`DATA_WIDTH-1:0]		NIOS_WRITE_DATA_I;
    reg 						NIOS_WRITE_DATA_EN_I;
    reg 						NIOS_WRITE_DATA_IDLE_I;
    wire						NIOS_WRITE_DATA_REQUEST_O;
	
    reg NIOS_READ_DATA_CLK_I;
    reg [`ADDRESS_WIDTH-1:0]	NIOS_READ_DATA_ADDRESS_I;
    wire [`DATA_WIDTH-1:0]	NIOS_CH0_READ_DATA_O;
    wire [`DATA_WIDTH-1:0]	NIOS_CH1_READ_DATA_O;
	
    wire [1:0]			NIOS_SAMPLE_FREQUENCY_O;

    // Bidirs


// Instantiate the UUT
    mp3_top uut (
	.HUFF_CLOCK_I(HUFF_CLOCK_I),
	.MAC_CLOCK_I(MAC_CLOCK_I),
	.RESETN_I(RESETN_I),
	
	// For NIOS MP3 data Input
	.NIOS_WRITE_DATA_CLK_I(NIOS_WRITE_DATA_CLK_I),
	.NIOS_WRITE_DATA_ADDRESS_I(NIOS_WRITE_DATA_ADDRESS_I),
	.NIOS_WRITE_DATA_I(NIOS_WRITE_DATA_I),
	.NIOS_WRITE_DATA_EN_I(NIOS_WRITE_DATA_EN_I),
	.NIOS_WRITE_DATA_REQUEST_O(NIOS_WRITE_DATA_REQUEST_O),
	.NIOS_WRITE_DATA_IDLE_I(NIOS_WRITE_DATA_IDLE_I),
		
	// For NIOS PCM sample output
	.NIOS_READ_DATA_CLK_I(NIOS_READ_DATA_CLK_I),
	.NIOS_READ_DATA_ADDRESS_I(NIOS_READ_DATA_ADDRESS_I),
	.NIOS_CH0_READ_DATA_O(NIOS_CH0_READ_DATA_O),
	.NIOS_CH1_READ_DATA_O(NIOS_CH1_READ_DATA_O),
	
	.NIOS_SAMPLE_FREQUENCY_O(NIOS_SAMPLE_FREQUENCY_O)
);


// the clock signals
parameter huff_cycle = 250; 	// 2 MHz
parameter mac_cycle = 125;	// 4 MHz
parameter nios_cycle = 10;	// 50 MHz

always begin
	#(huff_cycle);
	HUFF_CLOCK_I = 1'b0;
	#(huff_cycle);
	HUFF_CLOCK_I = 1'b1;
end

always begin
	#(mac_cycle);
	MAC_CLOCK_I = 1'b0;
	#(mac_cycle);
	MAC_CLOCK_I = 1'b1;
end

always begin
	#(nios_cycle);
	NIOS_WRITE_DATA_CLK_I = 1'b0;
	#(nios_cycle);
	NIOS_WRITE_DATA_CLK_I = 1'b1;
end

always begin
	#(nios_cycle);
	NIOS_READ_DATA_CLK_I = 1'b0;
	#(nios_cycle);
	NIOS_READ_DATA_CLK_I = 1'b1;
end

task master_reset;
// task for generating master reset for two clock cycles
begin
	wait (MAC_CLOCK_I !== 1'bx);
	@ (posedge MAC_CLOCK_I);
	RESETN_I = 1'b0;
	@ (posedge MAC_CLOCK_I);
	@ (posedge MAC_CLOCK_I);
	RESETN_I = 1'b1;
end
endtask

// Initialize Inputs
integer i;

// global variables
integer mp3_data_count;
integer frame, granule, channel, sample;
integer lframe, lgranule, lsample;
integer rframe, rgranule, rsample;
integer req_frame, req_granule;
integer alias_frame, alias_granule;
integer pcm_sample_counter;

// for reading MP3 data from files
reg [7:0] MP3_data [0:(`NUM_OF_BLOCK*512*2)-1];

task read_mp3_data;
// task for reading mp3 data from file for writing to HUFF ram from NIOS
integer i;
//integer j;
integer mp3_data_fp;
begin


	mp3_data_fp = $fopen("MP3/spot1.mp3","rb");

	if (mp3_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

//	i = $fread(mp3_data_fp, MP3_data[0], 0, (`NUM_OF_BLOCK*512));
	i = $fread(MP3_data, mp3_data_fp);
	
//	for(j = 0;j < 10; j = j + 2) $display("%d:%d", j/2, {MP3_data[j], MP3_data[j+1]});
	
	$fclose(mp3_data_fp);

	$display("Finish reading MP3 file with %d data...", i);
end
endtask

// for reading data from files										
integer unmatch_count [`NUM_OF_CHANNEL-1:0];
integer requantizer_unmatch_count [`NUM_OF_CHANNEL-1:0];
integer alias_unmatch_count [`NUM_OF_CHANNEL-1:0];
integer mdct_unmatch_count [`NUM_OF_CHANNEL-1:0];
integer icg_unmatch_count [`NUM_OF_CHANNEL-1:0];
integer ipg_unmatch_count [`NUM_OF_CHANNEL-1:0];
integer icg_b4_window_unmatch_count [`NUM_OF_CHANNEL-1:0];
integer ipg_b4_window_unmatch_count [`NUM_OF_CHANNEL-1:0];

reg [`DATA_WIDTH-1:0] requantizer_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][1023:0];
reg [`DATA_WIDTH-1:0] requantizer_block_type [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];
reg [1:0] requantizer_mode [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0];
reg [1:0] requantizer_mode_ext [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0];
reg [1:0] requantizer_frequency [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0];

task read_requantizer_data;
// task for reading anti-alias data from file for verification
integer frame, granule, channel, sample;
integer requantizer_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; // Line of text read from file 
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	requantizer_data_fp = $fopen("TXT/debug_requantize_output_16.txt", "r");

	if (requantizer_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(requantizer_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, requantizer_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, requantizer_data_fp);
         file_r = $fscanf(requantizer_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

				requantizer_data[frame][granule][channel][sample_num] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d sample_num=%d data=%h\n", frame, granule, channel, sample, sample_num, requantizer_data[frame][granule][channel][sample_num]);
				sample = sample + 1;
				if (sample_num == 575) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
		end
		file_c = $fgetc(requantizer_data_fp);
	end

	$fclose(requantizer_data_fp);

	$display("Finish reading requantizer data file...");
end
endtask

reg [`DATA_WIDTH-1:0] alias_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][`NUM_PCM_SAMPLE-1:0];
reg [`DATA_WIDTH-1:0] alias_block_type [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];
reg [`DATA_WIDTH-1:0] alias_mixed_block [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];
reg [`DATA_WIDTH-1:0] alias_window_switch [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0];

task read_alias_data;
// task for reading anti-alias data from file for verification
integer frame, granule, channel, sample;
integer alias_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	alias_data_fp = $fopen("TXT/debug_alias_input_16.txt", "r");

	if (alias_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(alias_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, alias_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, alias_data_fp);
         file_r = $fscanf(alias_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

			case (sample_num)
			-3: begin
				alias_mixed_block[frame][granule][channel] = sample_hex;
//				$write("fr=%d gr=%d ch=%d mixed_block=%d\n", frame, granule, channel, alias_mixed_block[frame][granule][channel]);
			end
			-2: begin
				alias_window_switch[frame][granule][channel] = sample_hex;
//				$write("fr=%d gr=%d ch=%d window_switch=%d\n", frame, granule, channel, alias_window_switch[frame][granule][channel]);
			end
			-1: begin
				alias_block_type[frame][granule][channel] = sample_hex;
//				$write("fr=%d gr=%d ch=%d block_type=%d\n", frame, granule, channel, alias_block_type[frame][granule][channel]);
			end
			default: begin
				alias_data[frame][granule][channel][sample_num] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d sample_num=%d data=%h\n", frame, granule, channel, sample, sample_num, alias_data[frame][granule][channel][sample_num]);
				sample = sample + 1;
				if (sample_num == 575) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
			end
			endcase
		end
		file_c = $fgetc(alias_data_fp);
	end

	$fclose(alias_data_fp);

	$display("Finish reading alias data file...");
end
endtask

reg [`DATA_WIDTH-1:0] icg_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][575:0];

task read_icg_data;
// task for reading icg data from file for verification
integer frame, granule, channel, sample;
integer icg_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; // Line of text read from file 
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	icg_data_fp = $fopen("TXT/debug_icg_16.txt", "r");

	if (icg_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(icg_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, icg_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, icg_data_fp);
         file_r = $fscanf(icg_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

				icg_data[frame][granule][channel][sample] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d data=%h\n", frame, granule, channel, sample, icg_data[frame][granule][channel][sample]);
				sample = sample + 1;
				if (sample == 576) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
		end
		file_c = $fgetc(icg_data_fp);
	end

	$fclose(icg_data_fp);

	$display("Finish reading icg data file...");
end
endtask

reg [`DATA_WIDTH-1:0] ipg_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][575:0];

task read_ipg_data;
// task for reading ipg data from file for verification
integer frame, granule, channel, sample;
integer ipg_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; // Line of text read from file 
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	ipg_data_fp = $fopen("TXT/debug_ipg_16.txt", "r");

	if (ipg_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(ipg_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, ipg_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, ipg_data_fp);
         file_r = $fscanf(ipg_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

				ipg_data[frame][granule][channel][sample] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d data=%h\n", frame, granule, channel, sample, ipg_data[frame][granule][channel][sample]);
				sample = sample + 1;
				if (sample == 576) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
		end
		file_c = $fgetc(ipg_data_fp);
	end

	$fclose(ipg_data_fp);

	$display("Finish reading ipg data file...");
end
endtask


reg [`DATA_WIDTH-1:0] icg_b4_window_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][287:0];

task read_icg_b4_window_data;
// task for reading icg data from file for verification
integer frame, granule, channel, sample;
integer icg_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; // Line of text read from file 
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	icg_data_fp = $fopen("TXT/debug_icg_b4_window_16.txt", "r");

	if (icg_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(icg_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, icg_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, icg_data_fp);
         file_r = $fscanf(icg_data_fp," %d: %h\n", sample_num, sample_hex);

				icg_b4_window_data[frame][granule][channel][sample] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d data=%h\n", frame, granule, channel, sample, icg_b4_window_data[frame][granule][channel][sample]);
				sample = sample + 1;
				if (sample == 288) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						while (alias_block_type[frame][granule][0] == 2) begin
							if (granule == 0) begin
								granule = 1;
							end else begin
								granule = 0;
								frame = frame + 1;
							end
						end
						channel = 0;
					end
				end
		end
		file_c = $fgetc(icg_data_fp);
	end

	$fclose(icg_data_fp);

	$display("Finish reading icg b4 window data file...");
end
endtask

reg [`DATA_WIDTH-1:0] ipg_b4_window_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][287:0];

task read_ipg_b4_window_data;
// task for reading ipg data from file for verification
integer frame, granule, channel, sample;
integer ipg_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; // Line of text read from file 
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	ipg_data_fp = $fopen("TXT/debug_ipg_b4_window_16.txt", "r");

	if (ipg_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(ipg_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, ipg_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, ipg_data_fp);
         file_r = $fscanf(ipg_data_fp," %d: %h\n", sample_num, sample_hex);

				ipg_b4_window_data[frame][granule][channel][sample] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d data=%h\n", frame, granule, channel, sample, ipg_b4_window_data[frame][granule][channel][sample]);
				sample = sample + 1;
				if (sample == 288) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						while (alias_block_type[frame][granule][0] == 2) begin
							if (granule == 0) begin
								granule = 1;
							end else begin
								granule = 0;
								frame = frame + 1;
							end
						end
						channel = 0;
					end
				end
		end
		file_c = $fgetc(ipg_data_fp);
	end

	$fclose(ipg_data_fp);

	$display("Finish reading ipg b4 window data file...");
end
endtask

reg [`DATA_WIDTH-1:0] mdct_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][17:0][63:0];

task read_mdct_data;
// task for reading mdct data from file for verification
integer frame, granule, channel, sb_count, sample;
integer mdct_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; // Line of text read from file 
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sb_count = 0;
	sample = 0;

	mdct_data_fp = $fopen("TXT/debug_mdct_output_16.txt", "r");

	if (mdct_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(mdct_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, mdct_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, mdct_data_fp);
         file_r = $fscanf(mdct_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

				mdct_data[frame][granule][channel][sb_count][sample_num] = sample_hex;
//				$write("fr=%d gr=%d ch=%d sample=%d sb_count=%d sample_num=%d data=%h\n", frame, granule, channel, sample, sb_count, sample_num, mdct_data[frame][granule][channel][sb_count][sample_num]);
				sample = sample + 1;
				if (sample_num == 63) sb_count = sb_count + 1;
				if (sample == 1152) begin
					$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
					sample = 0;
					sb_count = 0;
					if (channel == 0) begin
						channel = 1;
					end
					else begin
						if (granule == 0) begin
							granule = 1;
						end else begin
							granule = 0;
							frame = frame + 1;
						end
						channel = 0;
					end
				end
		end
		file_c = $fgetc(mdct_data_fp);
	end

	$fclose(mdct_data_fp);

	$display("Finish reading mdct data file...");
end
endtask


reg [`DATA_WIDTH-1:0] pcm_data [`NUM_OF_FRAME-1:0][`NUM_OF_GRANULE-1:0][`NUM_OF_CHANNEL-1:0][`NUM_PCM_SAMPLE-1:0];

task read_pcm_data;
// task for reading data from file for verification
integer frame, granule, channel, sample;
integer pcm_data_fp;
integer file_c, file_r;
reg [8*`MAX_LINE_LENGTH:0] line; /* Line of text read from file */
integer sample_num;
reg [`DATA_WIDTH-1:0] sample_dec, sample_hex;

begin
	frame = 0;
	granule = 0;
	channel = 0;
	sample = 0;

	pcm_data_fp = $fopen("TXT/debug_from_subsyn_16.txt", "r");

	if (pcm_data_fp == `NULL)	begin
		$display("file opening fail...");
		$stop(2);
	end else $display("Reading data file...");

	file_c = $fgetc(pcm_data_fp);
	for (frame = 0; frame < `NUM_OF_FRAME && file_c != `EOF; frame = frame) begin
		/* Check the first character for comment */
      if (file_c == "/") begin
			file_r = $fgets(line, pcm_data_fp);
      end else begin
	  		// Push the character back to the file then read the next time
         file_r = $ungetc(file_c, pcm_data_fp);
         file_r = $fscanf(pcm_data_fp," %d: %d (%h)\n", sample_num, sample_dec, sample_hex);

			pcm_data[frame][granule][channel][sample_num] = sample_hex;
			sample = sample + 1;
			if (sample_num == 575) begin
				$write("fr=%d gr=%d ch=%d: %d data read\n", frame, granule, channel, sample);
				sample = 0;
				if (channel == 0) begin
					channel = 1;
				end
				else begin
					if (granule == 0) begin
						granule = 1;
					end else begin
						granule = 0;
						frame = frame + 1;
					end
					channel = 0;
				end
			end
		end
		file_c = $fgetc(pcm_data_fp);
	end

	$fclose(pcm_data_fp);

	$display("Finish reading pcm data file...");
end
endtask

integer log_fp;

task check_data;
input frame, granule, channel, sample;
integer frame, granule, channel, sample;
input got_data;
reg [`DATA_WIDTH-1:0] got_data;
begin
	if (got_data != pcm_data[frame][granule][channel][sample]) begin
		log_fp = $fopen("TXT/data_log.txt", "a");
		$fwrite(log_fp, "Data unmatched at fr=%d gr=%d ch=%d sample=%d got=%h expected=%h\n", frame, granule, channel, sample, got_data, pcm_data[frame][granule][channel][sample]);
		$write("PCM unmatched at fr=%d gr=%d ch=%d sample=%d got=%h expected=%h\n", frame, granule, channel, sample, got_data, pcm_data[frame][granule][channel][sample]);
		$fclose(log_fp);
		unmatch_count[channel] = unmatch_count[channel] + 1;
	end
end
endtask

        initial begin
				// read expected data
				read_mp3_data;

`ifdef DEBUG_COMP_REQ
				read_requantizer_data;
`endif
`ifdef DEBUG_COMP_ANTI_IN
				read_alias_data;
`endif
`ifdef DEBUG_COMP_ICG_CH0
				read_icg_data;
`endif
`ifdef DEBUG_COMP_IPG_CH0
				read_ipg_data;
`endif
`ifdef DEBUG_COMP_ICG_B4_WIN_CH0
				read_icg_b4_window_data;
`endif
`ifdef DEBUG_COMP_IPG_B4_WIN_CH0
				read_ipg_b4_window_data;
`endif
`ifdef DEBUG_COMP_MDCT_CH0
				read_mdct_data;
`endif
`ifdef DEBUG_COMP_PCM_CH0
				read_pcm_data;
`endif

				frame = -2;
				channel = 0;
				granule = 0;
				sample = 0;

				lframe = `SKIP_FRAME;
				lgranule = 0;
				lsample = 0;

				rframe = `SKIP_FRAME;
				rgranule = 0;
				rsample = 0;

				req_frame = -1;
				req_granule = 0;

				alias_frame = `SKIP_FRAME;
				alias_granule = 0;

  				i = 0;
				unmatch_count[0] = 0;
				unmatch_count[1] = 0;
				requantizer_unmatch_count[0] = 0;
				requantizer_unmatch_count[1] = 0;
				alias_unmatch_count[0] = 0;
				alias_unmatch_count[1] = 0;
				mdct_unmatch_count[0] = 0;
				mdct_unmatch_count[1] = 0;
				icg_unmatch_count[0] = 0;
				icg_unmatch_count[1] = 0;				
				ipg_unmatch_count[0] = 0;
				ipg_unmatch_count[1] = 0;				
				icg_b4_window_unmatch_count[0] = 0;
				icg_b4_window_unmatch_count[1] = 0;
				ipg_b4_window_unmatch_count[0] = 0;
				ipg_b4_window_unmatch_count[1] = 0;
				pcm_sample_counter = 0;
				mp3_data_count = 0;

				$display("INFO: Simulation started...");

				// Generate reset signals
				master_reset;

				$display("INFO: Reset signals generated...");

				log_fp = $fopen("TXT/data_log.txt", "w");
				 
				//$fclose(log_fp);

				$display("INFO: Program started...");
	     end

/////////////////////////////// NIOS write data ////////////////////////////////////////////
	     reg NIOS_state;
	     reg [`ADDRESS_WIDTH-1:0] nios_write_counter;

	     //always @ (posedge NIOS_WRITE_DATA_CLK_I) begin
		//if (NIOS_WRITE_DATA_REQUEST_O == 1'b1)
		//$display("Write requested...");
	     //end
	     
	     always @ (posedge NIOS_WRITE_DATA_REQUEST_O) begin
		$display("Write requested...");
	     end
	     
	     // For writing to HUFF_DP_RAM from NIOS
	     always @ (posedge NIOS_WRITE_DATA_CLK_I) begin
		if (RESETN_I == 1'b0) begin
			nios_write_counter <= 0;
			NIOS_WRITE_DATA_ADDRESS_I <= 'h3FF;
			NIOS_WRITE_DATA_I <= 0;
			NIOS_WRITE_DATA_EN_I <= 0;
			NIOS_WRITE_DATA_IDLE_I <= 1;
			NIOS_state <= 0;

		end else begin
			case (NIOS_state)
			0: begin // wait for request
				NIOS_WRITE_DATA_IDLE_I <= 1;
				nios_write_counter <= 0;
				if (NIOS_WRITE_DATA_REQUEST_O == 1'b1) begin
					NIOS_state <= 1;
				end
			end
			default: begin // request detected
				if (nios_write_counter < 'd512) begin
					NIOS_WRITE_DATA_IDLE_I <= 0;
					//NIOS_WRITE_DATA_ADDRESS_I <= {~NIOS_WRITE_DATA_ADDRESS_I[`ADDRESS_WIDTH-1], nios_write_counter[`ADDRESS_WIDTH-2:0]};
					NIOS_WRITE_DATA_ADDRESS_I <= NIOS_WRITE_DATA_ADDRESS_I + 1;
					NIOS_WRITE_DATA_I <= {MP3_data[mp3_data_count],MP3_data[mp3_data_count+1]} ;
//					$display("%d: %d %d", NIOS_WRITE_DATA_ADDRESS_I, mp3_data_count, NIOS_WRITE_DATA_I);
					NIOS_WRITE_DATA_EN_I <= 1'b1;
					nios_write_counter <= nios_write_counter + 1;
					mp3_data_count <= mp3_data_count + 2;
				end else begin
					NIOS_state <= 0;
					NIOS_WRITE_DATA_EN_I <= 1'b0;
					$display("Write finished (%d)...\n", NIOS_WRITE_DATA_ADDRESS_I);
					//NIOS_WRITE_DATA_ADDRESS_I <= {~NIOS_WRITE_DATA_ADDRESS_I[`ADDRESS_WIDTH-1], nios_write_counter[`ADDRESS_WIDTH-2:0]};
				end
			end
			endcase
		end
	     end

/*	     
	     always @(posedge NIOS_WRITE_DATA_CLK_I) begin
		if (NIOS_WRITE_DATA_EN_I == 1'b1) begin
			$write("Write from NIOS: ad=%d di=%h\n", NIOS_WRITE_DATA_ADDRESS_I, NIOS_WRITE_DATA_I);
		end
	     end
*/	     
	     reg [9:0] read_delay_counter;
	     
/////////////////////////////// NIOS read data ////////////////////////////////////////////	     
		// NIOS_READ_DATA_CLK_I is 50 MHz
		// so slow down the read by 1000 to 50 kHz to mimic audio codec reading time
		
	     always @ (posedge NIOS_READ_DATA_CLK_I) begin
		if (RESETN_I == 1'b0) begin
			NIOS_READ_DATA_ADDRESS_I <= 'd450;
			read_delay_counter <= 0;
		end else begin
			if (read_delay_counter == 1000) begin
				//$display("Reading addr: %d\n", NIOS_READ_DATA_ADDRESS_I);
//				log_fp = $fopen("TXT/data_log.txt", "a");
//				$fwrite(log_fp,"sample %d: %d %h %h\n", pcm_sample_counter, NIOS_READ_DATA_ADDRESS_I-1, NIOS_CH0_READ_DATA_O, NIOS_CH1_READ_DATA_O);
//				$fclose(log_fp);
				pcm_sample_counter = pcm_sample_counter + 1;
				if (pcm_sample_counter == `NUM_OF_FRAME * `NUM_OF_GRANULE * `NUM_PCM_SAMPLE) begin
					$stop;
				end
				
				NIOS_READ_DATA_ADDRESS_I <= NIOS_READ_DATA_ADDRESS_I + 1;
				if (NIOS_READ_DATA_ADDRESS_I % 100 == 0) $display("Reading address %d\n", NIOS_READ_DATA_ADDRESS_I);
				read_delay_counter <= 0;
			end else begin
				read_delay_counter <= read_delay_counter + 1;
			end
		end
	     end

/////////////////////////////// Huffman ////////////////////////////////////////////

  		  always @ (posedge uut.HUFF_done) begin
		  	$write("Huffman finish...\n");
		  end

		  always @ (negedge uut.HUFF_done) begin
		  	$write("Huffman started...\n");
		  end
/*		  
		  always @ (posedge uut.HUFF_CLOCK_I) begin
			if (uut.CH0_MAC_RAM_we == 1'b1) begin
				$write("HUFF write L: ad=%d wd=%h\n", uut.MAC_RAM_write_address, uut.MAC_RAM_write_data);
			end
		  end
		  
		  always @ (posedge uut.HUFF_CLOCK_I) begin
			if (uut.CH1_MAC_RAM_we == 1'b1) begin
				$write("HUFF write R: ad=%d wd=%h\n", uut.MAC_RAM_write_address, uut.MAC_RAM_write_data);
			end
		  end		  
*/		  
`ifdef DEBUG_HUFF
			integer realframe;
			initial begin
				realframe = 0;
			end

			always @ (posedge uut.HUFF_CLOCK_I) begin
				if (uut.HUFFMANDECODER.state == 6)
				begin
					realframe = realframe + 1;
					$write("===========================================================\t\t\t\t\tcurrent frame: %d\n",realframe);
				end
			end

		  always @ (posedge uut.HUFF_CLOCK_I) begin
			  if (uut.HUFFMANDECODER.state == 11) begin
			    $write("HUFF write %d: \tglobal_gain = %d \tscalefac_compress = %d \tWSF = %d\n",
				   uut.HUFFMANDECODER.ram0_address,
				   uut.HUFFMANDECODER.ram0_datain[15:8],
				   uut.HUFFMANDECODER.ram0_datain[7:4],
					uut.HUFFMANDECODER.ram0_datain[3]
				 );
			    if ( uut.HUFFMANDECODER.ram0_datain[3] == 1)
				   $write("Block_type = %d \tMBF = %d\n",
					  uut.HUFFMANDECODER.ram0_datain[2:1],
					  uut.HUFFMANDECODER.ram0_datain[0]
				   );
			  end
		  end
`endif
/*
///////////////////////////// Requantizer ///////////////////////////////////////
  		  always @ (posedge uut.Requantizer_done) begin
		  	$write("Requantizer finish...\n");
			$write("Req unmatched (fr=%d, gr=%d): L: %d\n", req_frame, req_granule, requantizer_unmatch_count[0]);
			$write("Req unmatched (fr=%d, gr=%d): R: %d\n", req_frame, req_granule, requantizer_unmatch_count[1]);

			req_granule = uut.instance_Requantizer.granule_flag;
			if (req_granule == 0 && uut.instance_Requantizer.change_ch == 0) req_frame = req_frame + 1;
		  end

		  always @ (negedge uut.Requantizer_done) begin
		  	$write("Requantizer started...(fr=%d, gr=%d)\n", req_frame, req_granule);
//			$write("\t\t\t\t\t\t\t\t\t\t\tgranule_flag=%d\n", uut.instance_Requantizer.granule_flag);
		  end

		  always @ (posedge uut.instance_Requantizer.Stereo_dec_flag) begin
		  	$write("Stereo decoding started...(fr=%d gr=%d)\n", alias_frame, alias_granule);
			$write("Req unmatched (fr=%d, gr=%d): L: %d\n", req_frame, req_granule, requantizer_unmatch_count[0]);
			$write("Req unmatched (fr=%d, gr=%d): R: %d\n", req_frame, req_granule, requantizer_unmatch_count[1]);
			end

		  always @ (negedge uut.Stereo_start) begin
		  	$write("Stereo finish\n");
		  end

`ifdef DEBUG_REQ
		  always @ (posedge uut.HUFF_CLOCK_I) begin
			if (uut.instance_Requantizer.counter == 576 + 2 + 1 || uut.instance_Requantizer.counter == 584 + 2 + 1)
				$write("RAM global_gain = %d \tWSF = %d \tBlock_type = %d \tMBF = %d\n",
					uut.instance_Requantizer.REQ_DATA_I[15:8],
					uut.instance_Requantizer.REQ_DATA_I[3],
					uut.instance_Requantizer.REQ_DATA_I[2:1],
					uut.instance_Requantizer.REQ_DATA_I[0]
				);
		  end

		  always @ (posedge uut.HUFF_CLOCK_I) begin
		  		if (uut.instance_Requantizer.state == `State_ROM_Addr_comp && uut.instance_Requantizer.counter == 0) begin
					$write("freq=%d, mode=%d, mode_ext=%d, global_gain=%d block_type=%d scalefac_scale=%d preflag=%d	mixed_block_flag=%d window_switching_flag=%d zeros_pos_start=%d\n",
						uut.instance_Requantizer.FRQUANCY_I,
						uut.instance_Requantizer.MODE_I,
						uut.instance_Requantizer.MODE_EXT_I,
						uut.instance_Requantizer.global_gain,
						uut.instance_Requantizer.block_type,
						uut.instance_Requantizer.scalefac_scale,
						uut.instance_Requantizer.preflag,
						uut.instance_Requantizer.mixed_block_flag,
				      uut.instance_Requantizer.window_switching_flag,
						uut.instance_Requantizer.zeros_pos_start
					);
				end
		  end

		// for testing requantizer
		always @ (posedge uut.HUFF_CLOCK_I) begin
		   if (uut.instance_Requantizer.state == `State_reorder && uut.instance_Requantizer.counter !=576)begin
				 $write("count=%d  add0=%d add1=%d coeff=%h sca_fac=%d ch0_I=%h ch1_I=%h ch0_d=%h ch1_d=%h	\n",
				 uut.instance_Requantizer.counter,
				 uut.instance_Requantizer.DP_CH0_ADDRESS_O,
				 uut.instance_Requantizer.DP_CH1_ADDRESS_O,
				 uut.instance_Requantizer.Mult_in2,
				 uut.instance_Requantizer.scalefac_L,
				 uut.instance_Requantizer.REQ_CH0_DATA_I,
				 uut.instance_Requantizer.REQ_CH1_DATA_I,
				 uut.instance_Requantizer.DP_CH0_DATA_O,  
				 uut.instance_Requantizer.DP_CH1_DATA_O );
			end
		end

		 always @(posedge uut.HUFF_CLOCK_I)
			  if(uut.instance_Requantizer.state ==`State_value_Computation && uut.instance_Requantizer.Stereo_dec_flag==1'b0)
				  begin	
				  $display(" counter=%d\t  gain_exp=%h\t window=%d\t gain_index=%h in1=%d\t in2=%d\t mult_out=%d\t add_ch0=%d\t ch0=%h\t\t\t add_ch1=%d\t ch1=%h",
				  //$display(" i=%d\t add_ch0=%d\t ch0=%h add_ch1=%d\t ch1=%h",	
							uut.instance_Requantizer.counter,
							uut.instance_Requantizer.gain_exp,
							uut.instance_Requantizer.window, 
							uut.instance_Requantizer.gain_index,
							uut.instance_Requantizer.Mult_in1,
							uut.instance_Requantizer.Mult_in2,
							uut.instance_Requantizer.mult_out,
							uut.instance_Requantizer.REQ_CH0_ADDRESS_O,  				 
							uut.instance_Requantizer.REQ_CH0_DATA_O,
							uut.instance_Requantizer.REQ_CH1_ADDRESS_O,  				 
							uut.instance_Requantizer.REQ_CH1_DATA_O);
				  end
`endif

		wire [15:0] requantizer_data_output;
		assign requantizer_data_output = (uut.instance_Requantizer.change_ch == 0) ? 
														uut.instance_Requantizer.REQ_CH0_DATA_O
														: uut.instance_Requantizer.REQ_CH1_DATA_O;

`ifdef DEBUG_COMP_REQ
		  // for testing output of requantizer
		  always @ (posedge uut.HUFF_CLOCK_I) begin
			   if(uut.instance_Requantizer.state ==`State_value_Computation && uut.instance_Requantizer.Stereo_dec_flag==1'b0) begin
//					$write("Data: fr=%d gr=%d ch=%d addr=%d got=%h expected=%h\n", 
//					frame+2, 
//					granule, 
//					uut.instance_Requantizer.change_ch, 
//					uut.instance_Requantizer.counter, 
//					requantizer_data_output, 
//					requantizer_data[frame+2][granule][uut.instance_Requantizer.change_ch][uut.instance_Requantizer.counter]);

					if (requantizer_data[req_frame][req_granule][uut.instance_Requantizer.change_ch][uut.instance_Requantizer.counter] != requantizer_data_output) begin
						if (uut.instance_Requantizer.change_ch == 0) begin
							$write("L: ");
						end else begin
							$write("\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tR: ");
						end
						requantizer_unmatch_count[uut.instance_Requantizer.change_ch] = requantizer_unmatch_count[uut.instance_Requantizer.change_ch] + 1;

						$write("REQ unmatched: fr=%d gr=%d addr=%d got=%h expected=%h\n", 
						req_frame, 
						req_granule, 
						uut.instance_Requantizer.counter, 
						requantizer_data_output, 
						requantizer_data[req_frame][req_granule][uut.instance_Requantizer.change_ch][uut.instance_Requantizer.counter]);
					end
				end
		  end
`endif
*/

///////////////////////////////////// MAC unit //////////////////////////////////////////
			always @ (posedge uut.MAC_CLOCK_I) begin
			if (uut.CH0_MAC_start == 1'b1 && uut.MAC_done == 1'b0) begin
				$write("LEFT MAC started...(fr=%d, gr=%d)\n", lframe, lgranule);
				$write("\t\t\t\t\t\t\t\t\t\tstate=%d\n", uut.MAC_module.CH0_state);
				$write("\t\t\t\t\t\t\t\t\t\tblock_type=%d\n", uut.MAC_module.CH0_MAC_BLOCK_I);
//				$write("\t\t\t\t\t\t\t\t\t\thold=%d\n", uut.MAC_module.CH0_hold);
//				$write("\t\t\t\t\t\t\t\t\t\tcounter=%d\n", uut.MAC_module.CH0_counter);
//				$write("\t\t\t\t\t\t\t\t\t\taic_read_pointer=%d\n", uut.MAC_module.CH0_aic_read_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\taic_read_offset=%d\n", uut.MAC_module.CH0_aic_read_offset);
//				$write("\t\t\t\t\t\t\t\t\t\ticg_read_pointer=%d\n", uut.MAC_module.CH0_icg_read_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tantialias_write_pointer=%d\n", uut.MAC_module.CH0_antialias_write_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tantialias_read_pointer=%d\n", uut.MAC_module.CH0_antialias_read_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tbackup_pointer=%d\n", uut.MAC_module.CH0_backup_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tMAC_RAM_pointer=%d\n", uut.MAC_module.CH0_MAC_RAM_pointer);
//				$write("\n");			
			end
			if (uut.CH1_MAC_start == 1'b1 && uut.MAC_done == 1'b0) begin
				$write("RIGHT MAC started...(fr=%d, gr=%d)\n", rframe, rgranule);
				$write("\t\t\t\t\t\t\t\t\t\tstate=%d\n", uut.MAC_module.CH1_state);
				$write("\t\t\t\t\t\t\t\t\t\tblock_type=%d\n", uut.MAC_module.CH1_MAC_BLOCK_I);
//				$write("\t\t\t\t\t\t\t\t\t\thold=%d\n", uut.MAC_module.CH1_hold);
//				$write("\t\t\t\t\t\t\t\t\t\tcounter=%d\n", uut.MAC_module.CH1_counter);
//				$write("\t\t\t\t\t\t\t\t\t\taic_read_pointer=%d\n", uut.MAC_module.CH1_aic_read_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\taic_read_offset=%d\n", uut.MAC_module.CH1_aic_read_offset);
//				$write("\t\t\t\t\t\t\t\t\t\tantialias_write_pointer=%d\n", uut.MAC_module.CH1_antialias_write_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tantialias_read_pointer=%d\n", uut.MAC_module.CH1_antialias_read_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tbackup_pointer=%d\n", uut.MAC_module.CH1_backup_pointer);
//				$write("\t\t\t\t\t\t\t\t\t\tMAC_RAM_pointer=%d\n", uut.MAC_module.CH1_MAC_RAM_pointer);
			end
			if (uut.CH0_MAC_start == 1'b1) begin
				$write("\n");			
				$write("\t\t\t\t\t\t\t\t\t\tPCM_RAM_pointer=%d\n", uut.MAC_module.PCM_RAM_pointer);
			end
		  end

		  always @ (posedge uut.MAC_done) begin
		  	$write("MAC finished...wab=%d\n", uut.CH0_PCM_RAM_write_address_buffer);
		  end

///////////////////////////////////// Antialias ///////////////////////////////////////////
		  always @ (posedge uut.MAC_CLOCK_I) begin
		  	if (uut.MAC_module.CH0_state == `STATE_WAIT_OTHER_CHANNEL && ((uut.MAC_module.CH1_state == `STATE_WAIT_OTHER_CHANNEL) || (uut.MAC_module.CH1_state == `STATE_INITIAL))) begin
				$write("Anti-alias/IMDCT finished\n");
				$write("Alias unmatched: (fr=%d, gr=%d) L: %d\n", alias_frame, alias_granule, alias_unmatch_count[0]);
				$write("Alias unmatched: (fr=%d, gr=%d) R: %d\n", alias_frame, alias_granule, alias_unmatch_count[1]);
				$write("icg b4 window unmatched (fr=%d, gr=%d): L: %d\n", lframe, lgranule, icg_b4_window_unmatch_count[0]);
				$write("icg b4 window unmatched (fr=%d, gr=%d): R: %d\n", rframe, rgranule, icg_b4_window_unmatch_count[1]);
				$write("ipg b4 window unmatched (fr=%d, gr=%d): L: %d\n", lframe, lgranule, ipg_b4_window_unmatch_count[0]);
				$write("ipg b4 window unmatched (fr=%d, gr=%d): R: %d\n", rframe, rgranule, ipg_b4_window_unmatch_count[1]);
				$write("icg unmatched (fr=%d, gr=%d): L: %d\n", lframe, lgranule, icg_unmatch_count[0]);
				$write("icg unmatched (fr=%d, gr=%d): R: %d\n", rframe, rgranule, icg_unmatch_count[1]);
				$write("ipg unmatched (fr=%d, gr=%d): L: %d\n", lframe, lgranule, ipg_unmatch_count[0]);
				$write("ipg unmatched (fr=%d, gr=%d): R: %d\n", rframe, rgranule, ipg_unmatch_count[1]);
				if (alias_granule == 0) alias_granule = 1;
				else begin
					alias_granule = 0;
					alias_frame = alias_frame + 1;
				end
			end
		  end

`ifdef DEBUG_COMP_ANTI_IN
		  // for writing data to the DP RAM (MAC RAM) for data comparison in before antialiasing
			integer alias_state;
			integer alias_sb_count;
			integer alias_counter;
			integer left_alias_sample;
			integer right_alias_sample;
			reg [`DATA_WIDTH-1:0] CH0_ram_data [1023:0];
			reg [`DATA_WIDTH-1:0] CH1_ram_data [1023:0];
			reg Stereo_start_buf;
			reg [1:0] MP3_Info_CH0_block_type, MP3_Info_CH1_block_type;
				
			always @ (posedge uut.MAC_CLOCK_I or negedge uut.RESETN_I) begin
				if (uut.RESETN_I == 0) Stereo_start_buf <= 1'b0;
				else Stereo_start_buf <= uut.MAC_RAM_Idle;
			end

			always @ (posedge uut.MAC_CLOCK_I or negedge uut.RESETN_I) begin
		  		if (uut.RESETN_I == 0) begin
					alias_state <= 'd0;
					alias_sb_count <= 'd0;
					alias_counter <= 'd0;
					left_alias_sample <= 'd0;
					right_alias_sample <= 'd0;				
					MP3_Info_CH0_block_type <= 'd0;
					MP3_Info_CH1_block_type <= 'd0;
				end else begin
					if (uut.MAC_RAM_Idle == 1'b1 && Stereo_start_buf == 1'b0) begin
						$display("Start preparing antialising data for fr=%d gr=%d", alias_frame, alias_granule);
						
						MP3_Info_CH0_block_type <= alias_block_type[alias_frame][alias_granule][0];
						MP3_Info_CH1_block_type <= alias_block_type[alias_frame][alias_granule][1];

						alias_state <= 'd1;
						alias_sb_count <= 'd0;
						alias_counter <= 'd0;
						left_alias_sample <= 'd0;
						right_alias_sample <= 'd0;
					end else begin
						if (alias_counter + alias_sb_count * 32 == 'd1023) begin
							alias_state <= 'd0;
							alias_sb_count <= 'd0;
							alias_counter <= 'd0;
							left_alias_sample <= 'd0;
							right_alias_sample <= 'd0;
						end
					end

					if (alias_state == 'd0) begin
						alias_counter <= 'd0;
						alias_sb_count <= 'd0;
					end else begin
						if (alias_counter < 'd31) begin
							alias_counter <= alias_counter + 1;
						end else begin
							alias_sb_count <= alias_sb_count + 1;
							alias_counter <= 'd0;
						end
					end

					if (alias_state == 'd1) begin
						if (MP3_Info_CH0_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								if (left_alias_sample == `NUM_MAC_SAMPLE - 1) begin
									left_alias_sample <= 'd0;
								end else begin
									left_alias_sample <= left_alias_sample + 1;
								end
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									if (left_alias_sample == `NUM_MAC_SAMPLE-1) begin
										left_alias_sample <= 'd0;
									end else begin
										left_alias_sample <= left_alias_sample + 1;
									end
								end
							end
						end
					end

					if (alias_state == 'd1) begin
						if (MP3_Info_CH1_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								if (right_alias_sample == `NUM_MAC_SAMPLE - 1) begin
									right_alias_sample <= 'd0;
								end else begin
									right_alias_sample <= right_alias_sample + 1;
								end
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									if (right_alias_sample == `NUM_MAC_SAMPLE-1) begin
										right_alias_sample <= 'd0;
									end else begin
										right_alias_sample <= right_alias_sample + 1;
									end
								end
							end
						end
					end
		
					if (alias_state == 'd1) begin
						if (MP3_Info_CH0_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin								
								CH0_ram_data[alias_counter + alias_sb_count * 32]  <= alias_data[alias_frame][alias_granule][0][left_alias_sample];
/*
								$write("l: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH0_block_type,
											alias_sb_count, 
											alias_counter,
											alias_frame, 
											alias_granule, 
											left_alias_sample, 
											left_alias_sample,
											alias_counter + alias_sb_count * 32, 
											alias_data[alias_frame][alias_granule][0][left_alias_sample % `NUM_MAC_SAMPLE]
											);
*/
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									CH0_ram_data[alias_counter + alias_sb_count * 32] <= alias_data[alias_frame][alias_granule][0][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3];
/*
									$write("l: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH0_block_type,
											alias_sb_count, 
											alias_counter,
											alias_frame, 
											alias_granule, 
											left_alias_sample, 
											(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3,
											alias_counter + alias_sb_count * 32, 
											alias_data[alias_frame][alias_granule][0][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3]
											);
*/
								end
							end
						end
					end

					if (alias_state == 'd1) begin
						if (MP3_Info_CH1_block_type != 2) begin
							// writing normal block
							if (alias_counter % 16 < 9) begin
								CH1_ram_data[alias_counter + alias_sb_count * 32] <= alias_data[alias_frame][alias_granule][1][(right_alias_sample) % `NUM_MAC_SAMPLE];
/*
								$write("r: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH1_block_type,
											alias_sb_count, 
											alias_counter,
											alias_frame, 
											alias_granule, 
											right_alias_sample, 
											right_alias_sample,
											alias_counter + alias_sb_count * 32, 
											alias_data[alias_frame][alias_granule][1][right_alias_sample]
											);
*/
							end
						end else begin
							// writing short block
							if (alias_counter % 8 < 6) begin
								if (alias_counter < 24) begin
									CH1_ram_data[alias_counter + alias_sb_count * 32] <= alias_data[alias_frame][alias_granule][1][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3];
/*
									$write("r: bt=%d sb_count=%d counter=%d frame=%d gr=%d sample=%d addr1=%d addr2=%d di=%h\n", 
											MP3_Info_CH1_block_type,
											alias_sb_count, 
											alias_counter,
											alias_frame, 
											alias_granule, 
											right_alias_sample, 
											(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3,
											alias_counter + alias_sb_count * 32, 
											alias_data[alias_frame][alias_granule][1][(alias_sb_count*18)+(alias_counter/8)+(alias_counter % 8)*3]
											);
*/
								end
							end
						end
					end
				end
			end

		// compare data at input of antialias
		reg [9:0] CH0_address_buf;
		reg [9:0] CH1_address_buf;

		always @ (posedge uut.MAC_CLOCK_I) begin
			CH0_address_buf <= uut.MAC_module.CH0_MAC_RAM_ADDRESS_O;
		end
/*
		reg [3:0] state_buffer;
		
		always @ (posedge uut.MAC_CLOCK_I or negedge uut.RESETN_I) begin
			if (uut.RESETN_I == 1'b0) begin
				state_buffer <= uut.MAC_module.CH0_state;
			end else begin
				state_buffer <= uut.MAC_module.CH0_state;
				if (state_buffer != uut.MAC_module.CH0_state) 
					$display("State changed from %d to %d", state_buffer, uut.MAC_module.CH0_state);
			end
		end
*/
		
		always @(posedge uut.MAC_CLOCK_I) begin
		  	// input of antialias
			if (((uut.MAC_module.CH0_state == `STATE_LOAD_HALF_SB) && (uut.MAC_module.CH0_counter != 0))
			|| (uut.MAC_module.CH0_state == `STATE_ANTIALIAS)
			|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
			|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
			|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
			) 
			begin
/*
	   	$display("ANTI_IN: (fr=%d, gr=%d) CH0_state=%d counter=%2d en=%1d ra=%4d ab=%d do=%4h ed=%h", 
				alias_frame, alias_granule, 
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_counter, 
				uut.MAC_module.CH0_MAC_RAM_EN_O,
				uut.MAC_module.CH0_MAC_RAM_ADDRESS_O,
				CH0_address_buf,
				uut.MAC_module.CH0_MAC_RAM_DATA_I,
				CH0_ram_data[CH0_address_buf]);
*/
			if (CH0_ram_data[CH0_address_buf] != uut.MAC_module.CH0_MAC_RAM_DATA_I) begin
				alias_unmatch_count[0] = alias_unmatch_count[0] + 1;
				$display("L: Alias Data unmatched (fr=%d, gr=%d): addr=%d got=%h expected=%h", alias_frame, alias_granule, CH0_address_buf, uut.MAC_module.CH0_MAC_RAM_DATA_I, CH0_ram_data[CH0_address_buf]);
			end
			end
		end


		always @ (posedge uut.MAC_CLOCK_I) begin
			CH1_address_buf <= uut.MAC_module.CH1_MAC_RAM_ADDRESS_O;
		end

		always @(posedge uut.MAC_CLOCK_I) begin
		  	// input of antialias
			if (((uut.MAC_module.CH1_state == `STATE_LOAD_HALF_SB) && (uut.MAC_module.CH1_counter != 0))
			|| (uut.MAC_module.CH1_state == `STATE_ANTIALIAS)
			|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
			|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
			|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
			) 
			begin
/*
	   	$display("ANTI_IN: (fr=%d, gr=%d) CH1_state=%d counter=%2d en=%1d ra=%4d ab=%d do=%4h ed=%h", 
				alias_frame, alias_granule, 
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_counter, 
				uut.MAC_module.CH1_MAC_RAM_EN_O,
				uut.MAC_module.CH1_MAC_RAM_ADDRESS_O,
				CH1_address_buf,
				uut.MAC_module.CH1_MAC_RAM_DATA_I,
				CH1_ram_data[CH1_address_buf]);
*/
			if (CH1_ram_data[CH1_address_buf] != uut.MAC_module.CH1_MAC_RAM_DATA_I) begin
				alias_unmatch_count[1] = alias_unmatch_count[1] + 1;
				$display("\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\tR: Alias Data unmatched (fr=%d, gr=%d): addr=%d got=%h expected=%h", alias_frame, alias_granule, CH1_address_buf, uut.MAC_module.CH1_MAC_RAM_DATA_I, CH1_ram_data[CH1_address_buf]);
			end
			end
		end 
		
`endif

`ifdef DEBUG_ANTI_OUT
		  always @(posedge uut.MAC_CLOCK_I) begin
		  	// output of antialias
			if ((uut.MAC_module.CH0_state == `STATE_LOAD_HALF_SB || uut.MAC_module.CH0_state == `STATE_ANTIALIAS) 
			&& (uut.MAC_module.CH0_icg_wr_en == 1'b1) && lframe == 2)
			begin

	   	$display("ANTI_OUT: CH0_state=%d counter=%2d ra1=%4d ra2=%4d di=%4h da=%4h db=%4h mac=%1d wp=%4d we=%1b wa=%4d di=%4h", 
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_counter, 
				uut.MAC_module.CH0_aic_address, 
				uut.MAC_module.CH0_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH0_MAC_RAM_DATA_I,
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_antialias_write_pointer,
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_acc_out);

			end
		end

	 always @(posedge uut.MAC_CLOCK_I) begin
		  	// output of antialias
			if ((uut.MAC_module.CH1_state == `STATE_LOAD_HALF_SB || uut.MAC_module.CH1_state == `STATE_ANTIALIAS) 
			&& uut.MAC_module.CH1_icg_wr_en == 1'b1)
			begin

	   	$display("ANTI_OUT: CH1_state=%d counter=%2d ra1=%4d ra2=%4d da=%4h db=%4h mac=%1d wp=%4d we=%1b wa=%4d di=%4h", 
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_counter, 
				uut.MAC_module.CH1_aic_address, 
				uut.MAC_module.CH1_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_antialias_write_pointer,
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_acc_out);

			end
		end
`endif

///////////////////////////////////// IMDCT //////////////////////////////////////////////
`ifdef DEBUG_ICG_NORMAL_CH0
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for normal block
		if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE) && lframe == 2)
//		 && (uut.MAC_module.CH0_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t as=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t c=%d \t\t aa=%h \t\t ab=%h \t\t bt=%d \t\t wp=%h \t\t aic_ra=%d \t\t aic_do=%h \t\t ipg_ra=%d \t\t do=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_acc_select,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.instance_CH0_mult_accum.control,
				uut.MAC_module.instance_CH0_mult_accum.add_in1,
				uut.MAC_module.instance_CH0_mult_accum.add_in2,
				uut.MAC_module.CH0_MAC_BLOCK_I,
				uut.MAC_module.CH0_aic_window013_pointer,
				uut.MAC_module.CH0_aic_address,
				uut.MAC_module.CH0_aic_data_out,
				uut.MAC_module.CH0_ipg_address,
				uut.MAC_module.CH0_ipg_data_out,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				uut.MAC_module.CH0_icg_data_out);
	end
`endif

`ifdef DEBUG_ICG_NORMAL_CH1
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for normal block
		if ((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE) && lframe == 2)
//		 && (uut.MAC_module.CH1_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t as=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH1_counter,				
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_acc_select,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_icg_read_pointer,				
				uut.MAC_module.CH1_icg_mem_en, 
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_in,
				uut.MAC_module.CH1_icg_data_out);
	end
`endif

`ifdef DEBUG_ICG_SHORT_CH0
		always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for short block
		if (((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)))
//		 && (uut.MAC_module.CH0_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t addr=%d \t\t en=%d \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH0_MAC_RAM_EN_O,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				uut.MAC_module.CH0_icg_data_out);
	end
`endif

`ifdef DEBUG_ICG_SHORT_CH1
		always @ (posedge uut.MAC_module.MAC_CLOCK_I)
	// output of imdct for short block
		if (((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)))
//		 && (uut.MAC_module.CH1_icg_wr_en == 1'b1))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t addr=%d \t\t en=%d \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH1_counter,				
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_MAC_RAM_ADDRESS_O,
				uut.MAC_module.CH1_MAC_RAM_EN_O,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_icg_read_pointer,				
				uut.MAC_module.CH1_icg_mem_en, 
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_in,
				uut.MAC_module.CH1_icg_data_out);
	end
`endif

//////////////////////////////////// check for ICG before window ///////////////////////////////
`ifdef DEBUG_COMP_ICG_B4_WIN_CH0
	integer CH0_icg_count_b4_window;
	integer CH1_icg_count_b4_window;

	initial begin
		CH0_icg_count_b4_window = 0;
		CH1_icg_count_b4_window = 0;
	end

	reg [15:0] CH0_icg_b4_window_data_buffer;
	reg [15:0] CH1_icg_b4_window_data_buffer;

	always @ (posedge uut.MAC_module.MAC_CLOCK_I) begin
		if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE) && uut.MAC_module.CH0_counter == 1) begin
			CH0_icg_b4_window_data_buffer <= uut.MAC_module.CH0_icg_data_in;
		end
	end

	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of icg
	if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE) && (uut.MAC_module.CH0_icg_wr_en == 1'b1))
	begin
//	   	$display("LEFT icg b4 window: fr=%d, gr=%d, sn=%d, di=%h ed=%h", 
//				lframe, lgranule,
//				CH0_icg_count_b4_window/2,
//				CH0_icg_b4_window_data_buffer,
//				icg_b4_window_data[lframe][lgranule][0][CH0_icg_count_b4_window/2]
//				);

			if (icg_b4_window_data[lframe][lgranule][0][CH0_icg_count_b4_window/2] != CH0_icg_b4_window_data_buffer) begin
				icg_b4_window_unmatch_count[0] = icg_b4_window_unmatch_count[0] + 1;
				$display("L: icg b4 window Data unmatched (fr=%d, gr=%d, sn=%d): got=%h expected=%h", 
				lframe, lgranule,
				CH0_icg_count_b4_window/2,
				CH0_icg_b4_window_data_buffer,
				icg_b4_window_data[lframe][lgranule][0][CH0_icg_count_b4_window/2]
				);
			end

			if (CH0_icg_count_b4_window < 575) begin
				CH0_icg_count_b4_window = CH0_icg_count_b4_window + 1;
			end else begin
				CH0_icg_count_b4_window = 0;
			end
	end
`endif

`ifdef DEBUG_COMP_ICG_B4_WIN_CH1
	// CH1
	always @ (posedge uut.MAC_module.MAC_CLOCK_I) begin
		if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE) && uut.MAC_module.CH0_counter == 1) begin
			CH0_icg_b4_window_data_buffer <= uut.MAC_module.CH0_icg_data_in;
		end
	end

	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of icg
	if ((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE) && (uut.MAC_module.CH1_icg_wr_en == 1'b1))
	begin
//	   	$display("RIGHT icg b4 window: fr=%d, gr=%d, sn=%d, di=%h ed=%h", 
//				rframe, rgranule,
//				CH1_icg_count_b4_window/2,
//				CH1_icg_b4_window_data_buffer,
//				icg_b4_window_data[rframe][rgranule][1][CH1_icg_count_b4_window/2]
//				);

			if (icg_b4_window_data[rframe][rgranule][1][CH1_icg_count_b4_window/2] != CH1_icg_b4_window_data_buffer) begin
				icg_b4_window_unmatch_count[1] = icg_b4_window_unmatch_count[1] + 1;
				$display("R: icg b4 window Data unmatched (fr=%d, gr=%d, sn=%d): got=%h expected=%h", 
				rframe, rgranule,
				CH1_icg_count_b4_window/2,
				CH1_icg_b4_window_data_buffer,
				icg_b4_window_data[rframe][rgranule][1][CH1_icg_count_b4_window/2]
				);
			end

			if (CH1_icg_count_b4_window < 575) begin
				CH1_icg_count_b4_window = CH1_icg_count_b4_window + 1;
			end else begin
				CH1_icg_count_b4_window = 0;
			end
	end
`endif

////////////////////////////////////// check for ICG //////////////////////////
	integer CH0_icg_count;
	integer CH1_icg_count;

	initial begin
		CH0_icg_count = 0;
		CH1_icg_count = 0;
	end

`ifdef DEBUG_COMP_ICG_CH0
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of icg
	if (((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE)
   	|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND))
		 && (uut.MAC_module.CH0_icg_wr_en == 1'b1))
	begin
/*
	   	$display("LEFT icg: fr=%d, gr=%d, sn=%d, ad=%d di=%h ed=%h", 
				lframe, lgranule,
				CH0_icg_count,
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				icg_data[lframe][lgranule][0][uut.MAC_module.CH0_icg_address]
				);
*/
			if (icg_data[lframe][lgranule][0][uut.MAC_module.CH0_icg_address] != uut.MAC_module.CH0_icg_data_in) begin
				icg_unmatch_count[0] = icg_unmatch_count[0] + 1;
				$display("L: icg Data unmatched (fr=%d, gr=%d, sn=%d): addr=%d got=%h expected=%h", 
				lframe, lgranule,
				CH0_icg_count,
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				icg_data[lframe][lgranule][0][uut.MAC_module.CH0_icg_address]
				);
			end

			if (CH0_icg_count < 575) begin
				CH0_icg_count = CH0_icg_count + 1;
			end else begin
				CH0_icg_count = 0;
			end
	end
`endif

`ifdef DEBUG_COMP_ICG_CH1
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of icg
	if (((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_CURRENT_GRANULE)
   	|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND))
		 && (uut.MAC_module.CH1_icg_wr_en == 1'b1))
	begin

//	   	$display("RIGHT icg: fr=%d, gr=%d, sn=%d, ad=%d di=%h ed=%h", 
//				rframe, rgranule,
//				CH1_icg_count,
//				uut.MAC_module.CH1_icg_address,
//				uut.MAC_module.CH1_icg_data_in,
//				icg_data[rframe][rgranule][1][uut.MAC_module.CH1_icg_address]
//				);

			if (icg_data[rframe][rgranule][1][uut.MAC_module.CH1_icg_address] != uut.MAC_module.CH1_icg_data_in) begin
				icg_unmatch_count[1] = icg_unmatch_count[1] + 1;
				$display("R: icg Data unmatched (fr=%d, gr=%d, sn=%d): addr=%d got=%h expected=%h", 
				rframe, rgranule,
				CH1_icg_count,
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_in,
				icg_data[rframe][rgranule][1][uut.MAC_module.CH1_icg_address]
				);
			end

			if (CH1_icg_count < 575) begin
				CH1_icg_count = CH1_icg_count + 1;
			end else begin
				CH1_icg_count = 0;
			end
	end
`endif
////////////////////////////////////////////////// check for IPG before window //////////////////////////////
	integer CH0_ipg_count_b4_window;
	integer CH1_ipg_count_b4_window;

	initial begin
		CH0_ipg_count_b4_window = 0;
		CH1_ipg_count_b4_window = 0;
	end

	reg [15:0] CH0_ipg_b4_window_data_buffer;
	reg [15:0] CH1_ipg_b4_window_data_buffer;

`ifdef DEBUG_COMP_IPG_B4_WIN_CH0
	always @ (posedge uut.MAC_module.MAC_CLOCK_I) begin
		if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE) && uut.MAC_module.CH0_counter == 1) begin
			CH0_ipg_b4_window_data_buffer <= uut.MAC_module.CH0_ipg_data_in;
		end
	end

	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of ipg
	if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE) && (uut.MAC_module.CH0_ipg_wr_en == 1'b1))
	begin
//	   	$display("LEFT ipg b4 window: fr=%d, gr=%d, sn=%d, di=%h ed=%h", 
//				lframe, lgranule,
//				CH0_ipg_count_b4_window/2,
//				CH0_ipg_b4_window_data_buffer,
//				ipg_b4_window_data[lframe][lgranule][0][CH0_ipg_count_b4_window/2]
//				);

			if (ipg_b4_window_data[lframe][lgranule][0][CH0_ipg_count_b4_window/2] != CH0_ipg_b4_window_data_buffer) begin
				ipg_b4_window_unmatch_count[0] = ipg_b4_window_unmatch_count[0] + 1;
				$display("L: ipg b4 window Data unmatched (fr=%d, gr=%d, sn=%d): got=%h expected=%h", 
				lframe, lgranule,
				CH0_ipg_count_b4_window/2,
				CH0_ipg_b4_window_data_buffer,
				ipg_b4_window_data[lframe][lgranule][0][CH0_ipg_count_b4_window/2]
				);
			end

			if (CH0_ipg_count_b4_window < 575) begin
				CH0_ipg_count_b4_window = CH0_ipg_count_b4_window + 1;
			end else begin
				CH0_ipg_count_b4_window = 0;
			end
	end
`endif

`ifdef DEBUG_COMP_IPG_B4_WIN_CH1
	always @ (posedge uut.MAC_module.MAC_CLOCK_I) begin
		if ((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE) && uut.MAC_module.CH1_counter == 1) begin
			CH1_ipg_b4_window_data_buffer <= uut.MAC_module.CH1_ipg_data_in;
		end
	end

	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of ipg
	if ((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE) && (uut.MAC_module.CH1_ipg_wr_en == 1'b1))
	begin
//	   	$display("RIGHT ipg b4 window: fr=%d, gr=%d, sn=%d, di=%h ed=%h", 
//				rframe, rgranule,
//				CH1_ipg_count_b4_window/2,
//				CH1_ipg_b4_window_data_buffer,
//				ipg_b4_window_data[rframe][rgranule][1][CH1_ipg_count_b4_window/2]
//				);

			if (ipg_b4_window_data[rframe][rgranule][1][CH1_ipg_count_b4_window/2] != CH1_ipg_b4_window_data_buffer) begin
				ipg_b4_window_unmatch_count[1] = ipg_b4_window_unmatch_count[1] + 1;
				$display("R: ipg b4 window Data unmatched (fr=%d, gr=%d, sn=%d): got=%h expected=%h", 
				rframe, rgranule,
				CH1_ipg_count_b4_window/2,
				CH1_ipg_b4_window_data_buffer,
				ipg_b4_window_data[rframe][rgranule][1][CH1_ipg_count_b4_window/2]
				);
			end

			if (CH1_ipg_count_b4_window < 575) begin
				CH1_ipg_count_b4_window = CH1_ipg_count_b4_window + 1;
			end else begin
				CH1_ipg_count_b4_window = 0;
			end
	end
`endif

////////////////////////////////////////////////// check for IPG //////////////////////////////

	integer CH0_ipg_count;
	integer CH1_ipg_count;

	initial begin
		CH0_ipg_count = 0;
		CH1_ipg_count = 0;
	end

`ifdef DEBUG_IPG_NORMAL_CH0
	always @ (posedge uut.MAC_module.MAC_CLOCK_I) begin
		if ((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE)) begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t as=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_acc_select,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_ipg_mem_en, 
				uut.MAC_module.CH0_ipg_wr_en, 
				uut.MAC_module.CH0_ipg_address,
				uut.MAC_module.CH0_ipg_data_in,
				uut.MAC_module.CH0_ipg_data_out);			
		end
	end
`endif

`ifdef DEBUG_IPG_NORMAL_CH1
	always @ (posedge uut.MAC_module.MAC_CLOCK_I) begin
		if ((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE)) begin
	   	$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t as=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH1_counter,				
				uut.MAC_module.CH1_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_acc_select,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_icg_read_pointer,				
				uut.MAC_module.CH1_ipg_mem_en, 
				uut.MAC_module.CH1_ipg_wr_en, 
				uut.MAC_module.CH1_ipg_address,
				uut.MAC_module.CH1_ipg_data_in,
				uut.MAC_module.CH1_ipg_data_out);			
		end
	end
`endif

`ifdef DEBUG_COMP_IPG_CH0
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of ipg
	if (((uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE)
   	|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_LOAD_IPG)
		|| (uut.MAC_module.CH0_state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD))
		 && (uut.MAC_module.CH0_ipg_wr_en == 1'b1))
	begin

//	   	$display("LEFT ipg: fr=%d, gr=%d, sn=%d, ad=%d di=%h ed=%h", 
//				lframe, lgranule,
//				CH0_ipg_count,
//				uut.MAC_module.CH0_ipg_address,
//				uut.MAC_module.CH0_ipg_data_in,
//				ipg_data[lframe][lgranule][0][uut.MAC_module.CH0_ipg_address]
//				);

			if (ipg_data[lframe][lgranule][0][uut.MAC_module.CH0_ipg_address] != uut.MAC_module.CH0_ipg_data_in) begin
				ipg_unmatch_count[0] = ipg_unmatch_count[0] + 1;
				$display("L: ipg Data unmatched (fr=%d, gr=%d, sn=%d): addr=%d got=%h expected=%h", 
				lframe, lgranule,
				CH0_ipg_count,
				uut.MAC_module.CH0_ipg_address,
				uut.MAC_module.CH0_ipg_data_in,
				ipg_data[lframe][lgranule][0][uut.MAC_module.CH0_ipg_address]
				);
			end

			if (CH0_ipg_count < 575) begin
				CH0_ipg_count = CH0_ipg_count + 1;
			end else begin
				CH0_ipg_count = 0;
			end
	end
`endif

`ifdef DEBUG_COMP_IPG_CH1
	// CH1
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of ipg
	if (((uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK0_PREVIOUS_GRANULE)
   	|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_LOAD_ICG)
		|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST)
		|| (uut.MAC_module.CH1_state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND))
		 && (uut.MAC_module.CH1_ipg_wr_en == 1'b1))
	begin

//	   	$display("RIGHT ipg: fr=%d, gr=%d, sn=%d, ad=%d di=%h ed=%h", 
//				rframe, rgranule,
//				CH1_ipg_count,
//				uut.MAC_module.CH1_ipg_address,
//				uut.MAC_module.CH1_ipg_data_in,
//				ipg_data[rframe][rgranule][1][uut.MAC_module.CH1_ipg_address]
//				);

			if (ipg_data[rframe][rgranule][1][uut.MA  C_module.CH1_ipg_address] != uut.MAC_module.CH1_ipg_data_in) begin
				ipg_unmatch_count[1] = ipg_unmatch_count[1] + 1;
				$display("R: ipg Data unmatched (fr=%d, gr=%d, sn=%d): addr=%d got=%h expected=%h", 
				rframe, rgranule,
				CH1_ipg_count,
				uut.MAC_module.CH1_ipg_address,
				uut.MAC_module.CH1_ipg_data_in,
				ipg_data[rframe][rgranule][1][uut.MAC_module.CH1_ipg_address]
				);
			end

			if (CH1_ipg_count < 575) begin
				CH1_ipg_count = CH1_ipg_count + 1;
			end else begin
				CH1_ipg_count = 0;
			end
	end
`endif

///////////////////////////////////// Frequency inversion //////////////////////////////////////////////
`ifdef DEBUG_FREQ_INV
	always @ (posedge uut.MAC_module.MAC_CLOCK_I)
//		if (uut.MAC_module.CH0_state == `STATE_IMDCT_FREQ_INV)
		if (uut.MAC_module.CH0_icg_wr_en == 1'b1)
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t di=%h \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_in,
				uut.MAC_module.CH0_icg_data_out);
	end
`endif

///////////////////////////////////// MDCT //////////////////////////////////////////////
`ifdef DEBUG_MDCT_CH0
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// input to MDCT
	if ((uut.MAC_module.CH0_state == `STATE_POLYPHASE_MDCT) && (uut.MAC_module.CH0_counter[9:5] == 0))
	begin
	   	$display("LEFT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.CH0_icg_read_pointer,				
				uut.MAC_module.CH0_icg_mem_en, 
				uut.MAC_module.CH0_icg_wr_en, 
				uut.MAC_module.CH0_icg_address,
				uut.MAC_module.CH0_icg_data_out);
	end
`endif

`ifdef DEBUG_MDCT_CH1
always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// input to MDCT
	if ((uut.MAC_module.CH0_state == `STATE_POLYPHASE_MDCT) && (uut.MAC_module.CH0_counter[9:5] == 0) && (uut.MAC_module.CH1_state == `STATE_WAIT_OTHER_CHANNEL))
	begin
	   	$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t icg_rp=%d \t\t en=%d \t\t we=%d \t\t ra=%d \t\t do=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.CH1_icg_read_pointer,				
				uut.MAC_module.CH1_icg_mem_en, 
				uut.MAC_module.CH1_icg_wr_en, 
				uut.MAC_module.CH1_icg_address,
				uut.MAC_module.CH1_icg_data_out);
	end
`endif

	integer lmdct_sb_count;
	integer lmdct_count;
	integer rmdct_sb_count;
	integer rmdct_count;

	initial begin
		lmdct_sb_count = 0;
		lmdct_count = 0;
		rmdct_sb_count = 0;
		rmdct_count = 0;
	end

`ifdef DEBUG_COMP_MDCT_CH0
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output to MDCT
	if (uut.MAC_module.CH0_psb_wr_en == 1)
	begin

//	   	$display("LEFT MDCT: fr=%d, gr=%d, sb=%d, sn=%d, ad=%d di=%h ed=%h", 
//				lframe, lgranule,
//				lmdct_sb_count,
//				lmdct_count,
//				uut.MAC_module.CH0_psb_address,
//				uut.MAC_module.CH0_psb_data_in,
//				mdct_data[lframe][lgranule][0][lmdct_sb_count][lmdct_count]
//				);

			if (mdct_data[lframe][lgranule][0][lmdct_sb_count][uut.MAC_module.CH0_psb_address % 64] != uut.MAC_module.CH0_psb_data_in) begin
				mdct_unmatch_count[0] = mdct_unmatch_count[0] + 1;
				$display("L: MDCT Data unmatched (fr=%d, gr=%d, sb=%d, sn=%d): addr=%d got=%h expected=%h", 
				lframe, lgranule,
				lmdct_sb_count,
				lmdct_count,
				uut.MAC_module.CH0_psb_address,
				uut.MAC_module.CH0_psb_data_in,
				mdct_data[lframe][lgranule][0][lmdct_sb_count][uut.MAC_module.CH0_psb_address % 64]
				);
			end

			if (lmdct_count < 63) begin
				lmdct_count = lmdct_count + 1;
			end else begin
				lmdct_count = 0;
				if (lmdct_sb_count < 17) begin
					lmdct_sb_count = lmdct_sb_count + 1;
				end else begin
					lmdct_sb_count = 0;
					$write("MDCT unmatched: (fr=%d, gr=%d) L: %d\n", lframe, lgranule, mdct_unmatch_count[0]);
				end
			end
	end
`endif

`ifdef DEBUG_COMP_MDCT_CH1
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output to MDCT
	if (uut.MAC_module.CH1_psb_wr_en == 1)
	begin
//	   	$display("RIGHT MDCT: fr=%d, gr=%d, sb=%d, sn=%d, ad=%d di=%h ed=%h", 
//				rframe, rgranule,
//				rmdct_sb_count,
//				rmdct_count,
//				uut.MAC_module.CH1_psb_address,
//				uut.MAC_module.CH1_psb_data_in,
//				mdct_data[rframe][rgranule][1][rmdct_sb_count][rmdct_count]
//				);

			if (mdct_data[rframe][rgranule][1][rmdct_sb_count][uut.MAC_module.CH1_psb_address % 64] != uut.MAC_module.CH1_psb_data_in) begin
				mdct_unmatch_count[1] = mdct_unmatch_count[1] + 1;
				$display("R: MDCT Data unmatched (fr=%d, gr=%d, sb=%d, sn=%d): addr=%d got=%h expected=%h", 
				rframe, rgranule,
				rmdct_sb_count,
				rmdct_count,
				uut.MAC_module.CH1_psb_address,
				uut.MAC_module.CH1_psb_data_in,
				mdct_data[rframe][rgranule][1][rmdct_sb_count][uut.MAC_module.CH1_psb_address % 64]
				);
			end

			if (rmdct_count < 63) begin
				rmdct_count = rmdct_count + 1;
			end else begin
				rmdct_count = 0;
				if (rmdct_sb_count < 17) begin
					rmdct_sb_count = rmdct_sb_count + 1;
				end else begin
					rmdct_sb_count = 0;
					$write("MDCT unmatched: (fr=%d, gr=%d) R: %d\n", rframe, rgranule, mdct_unmatch_count[1]);
				end
			end
	end
`endif

///////////////////////////////////// Windowing //////////////////////////////////////////////
`ifdef DEBUG_POLY_WINDOW_CH0
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of polyphase windowing
	if (uut.MAC_module.CH0_state == `STATE_POLYPHASE_WINDOW && lframe == 2)
//	&& uut.MAC_module.CH0_PCM_RAM_WE_O)
	begin
	 	$display("LEFT : counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t ca=%d \t\t cd=%h \t\t ra=%d \t\t di=%h \t\t we=%d \t\t wa=%d \t\t wd=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH0_add_accum_control,
				uut.MAC_module.CH0_bypass_mult,
				uut.MAC_module.CH0_control_mux_a, 
				uut.MAC_module.CH0_control_mux_b, 
				uut.MAC_module.CH0_mult_accum_data_a,
				uut.MAC_module.CH0_mult_accum_data_b,
				uut.MAC_module.pwc_address,
				uut.MAC_module.pwc_data_out,
				uut.MAC_module.CH0_psb_address,
				uut.MAC_module.CH0_psb_data_out,
				uut.MAC_module.CH0_PCM_RAM_WE_O,
				uut.MAC_module.CH0_PCM_RAM_ADDRESS_O,
				uut.MAC_module.CH0_PCM_RAM_DATA_O
				);
	end
`endif

`ifdef DEBUG_POLY_WINDOW_CH1
	always @(posedge uut.MAC_module.MAC_CLOCK_I) 
	// output of polyphase windowing
	if (uut.MAC_module.CH0_state == `STATE_POLYPHASE_WINDOW && rframe == 2)
	begin
		$display("RIGHT: counter=%d \t\t state=%d \t\t mac=%d \t\t b=%d \t\t ca=%d \t\t cb=%d \t\t da=%h \t\t db=%h \t\t ca=%d \t\t cd=%h \t\t ra=%d \t\t di=%h \t\t we=%d \t\t wa=%d \t\t wd=%h", 
				uut.MAC_module.CH0_counter,				
				uut.MAC_module.CH0_state, 
				uut.MAC_module.CH1_add_accum_control,
				uut.MAC_module.CH1_bypass_mult,
				uut.MAC_module.CH1_control_mux_a, 
				uut.MAC_module.CH1_control_mux_b, 
				uut.MAC_module.CH1_mult_accum_data_a,
				uut.MAC_module.CH1_mult_accum_data_b,
				uut.MAC_module.pwc_address,
				uut.MAC_module.pwc_data_out,
				uut.MAC_module.CH1_psb_address,
				uut.MAC_module.CH1_psb_data_out,
				uut.MAC_module.CH1_PCM_RAM_WE_O,
				uut.MAC_module.CH1_PCM_RAM_ADDRESS_O,
				uut.MAC_module.CH1_PCM_RAM_DATA_O
				);
	end
`endif

///////////////////////////////////// PCM //////////////////////////////////////////////
`ifdef DEBUG_COMP_PCM_CH0
			// for behavioral
			always @ (posedge uut.MAC_CLOCK_I) begin
		  		// Print read PCM sample
		  		if (uut.CH0_PCM_RAM_we == 1'b1) begin
					if (lsample % 100 == 0) $write("L: Sample written: %d\n", lsample);

					if (lframe >= 0) begin
//						$write("WRITE_PCM L: (%d, %d, %d) ad=%d got=%h expected=%h\n", lframe, lgranule, lsample, uut.CH0_PCM_RAM_write_address, uut.CH0_PCM_RAM_write_data, pcm_data[lframe][lgranule][0][lsample]);
						check_data(lframe, lgranule, 0, lsample, uut.CH0_PCM_RAM_write_data);
					end

					lsample = lsample+1;

					if (lsample == `NUM_PCM_SAMPLE) begin
						if (lframe >= 0) begin
							//$write("Last left value in frame: got=%h expected=%h\n", uut.ac97_module.CH0_PCM_DATA_I, pcm_data[frame][granule][0][sample-1]);
							//$write("Last right value in frame: got=%h expected=%h\n", uut.ac97_module.CH1_PCM_DATA_I, pcm_data[frame][granule][1][sample-1]);
							$write("PCM unmatched (fr=%d, gr=%d): L: %d\n", lframe, lgranule, unmatch_count[0]);
							if (lgranule == 0) lgranule = 1;
							else begin
								lgranule = 0;
								lframe = lframe + 1;
							end
						end else
						lframe = lframe + 1;
						lsample = 0;
					end
				end
		  end
`endif

`ifdef DEBUG_COMP_PCM_CH1
		  	always @ (posedge uut.MAC_CLOCK_I) begin
		  		// Print read PCM sample
		  		if (uut.CH1_PCM_RAM_we == 1'b1) begin
					if (rsample % 100 == 0) $write("R: Sample written: %d\n", rsample);

					if (rframe >= 0) begin
//						$write("WRITE_PCM R: (%d, %d, %d) got=%h expected=%h\n", rframe, rgranule, rsample, uut.CH1_PCM_RAM_write_data, pcm_data[rframe][rgranule][1][rsample]);
						check_data(rframe, rgranule, 1, rsample, uut.CH1_PCM_RAM_write_data);
					end

					rsample = rsample+1;

					if (rsample == `NUM_PCM_SAMPLE) begin
						if (rframe >= 0) begin
							$write("PCM unmatched (fr=%d, gr=%d): R: %d\n", rframe, rgranule, unmatch_count[1]);
							if (rgranule == 0) rgranule = 1;
							else begin
								rgranule = 0;
								rframe = rframe + 1;
							end
						end else
						rframe = rframe + 1;
						rsample = 0;
					end
				end
		  end
`endif
/*
		  always @ (posedge AC97_SYNCH_O) begin
		  		// Print read PCM sample
		  		if (i > 15) begin
					if (sample % 100 == 0) $write("Sample read: %d\n", sample);

					sample = sample+1;

					if (sample == `NUM_PCM_SAMPLE) begin
						if (frame >= 0) begin
							if (granule == 0) granule = 1;
							else begin
								granule = 0;
								frame = frame + 1;
							end
						end else
						frame = frame + 1;
						sample = 0;
					end

					if (frame == `NUM_OF_FRAME) begin
						# 1000;
						log_fp = $fopen("TXT/data_log.txt", "a");
						$fwrite(log_fp, "\n\nTotal unmatch count: ch0=%d ch1=%d\n", unmatch_count[0], unmatch_count[1]);
						$write("\n\nTotal unmatch count: ch0=%d ch1=%d\n", unmatch_count[0], unmatch_count[1]);
						$fclose(log_fp);
						$stop(2);
					end
				end
		  end
		  
		  always @ (posedge AC97_SYNCH_O) begin
		  		if (i > 15) #10;
				else 
				i = i+1;
		
//					$write("INFO: %d: %d\n", i, uut.MAC_module_MAC_RAM_ADDRESS_O);
				#10;
		  end
*/
////////////////////////////////////////// Other /////////////////////////////
/*
	always @ (uut.MAC_module.state) begin
	if ((uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_FIRST) 
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_SECOND)
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_COMPUTE_THIRD)
	|| (uut.MAC_module.state == `STATE_IMDCT_BLOCK2_LOAD_IPG))
		$display("\t");
	end
*/
//reg [15:0] left_array[1023:0];
//reg [15:0] right_array[1023:0];
/*
always @(posedge HUFF_CLOCK_I)
begin
  if (uut.HUFFMANDECODER.ch == 1'b1)
  begin
    if (uut.HUFFMANDECODER.mac_left_we == 1'b1)
    begin
      //left_array[uut.MAC_RAM_write_address] = uut.MAC_RAM_write_data;
      $write("Writing Left: addr=%d data=%x\n",uut.MAC_RAM_write_address,uut.MAC_RAM_write_data);
    end
    if (uut.HUFFMANDECODER.mac_right_we == 1'b1)
    begin
      //right_array[uut.MAC_RAM_write_address] = uut.MAC_RAM_write_data;
      $write("Writing Right: addr=%d data=%x\n",uut.MAC_RAM_write_address,uut.MAC_RAM_write_data);
    end
  end
end
*/
// HUFFMAN Testbench

        integer endsim;
        integer dfile,rfile,sfile;
        integer mismatch;
        integer decoded_sample;
        integer read_decoded_sample;
        integer requantized_sample;
        integer read_requantized_sample;
        integer stereo_sample;
        integer read_stereo_sample;
        integer leftram[576];
        integer rightram[576];

        initial begin
            dfile = $fopen("decoded_samples.txt","r");
            rfile = $fopen("requantized_samples.txt","r");
            sfile = $fopen("stereo_samples.txt","r");
            mismatch = 0;
        end
        
        `define COMPARE_HUFFMAN 1
        //`define COMPARE_STEREO 1
        `define SHOW_MISMATCH 1
        //`define SHOW_SCALEFAC 1
        //`define COMPARE_REORDER 1
        `define MISMATCH_BEFORE_PAUSE 40
        `define MISMATCH_TOLERANCE 0
        
        always @ (posedge HUFF_CLOCK_I)
        begin
          case (uut.HUFFMANDECODER.stage)
          `STAGE_SIDEINFO:
          begin
            case (uut.HUFFMANDECODER.state)
            `STATE_SIDEINFO_ADDRESS:
            begin
              $write("===================sideinfo gr%d ch%d\n", uut.HUFFMANDECODER.gr, uut.HUFFMANDECODER.ch);
            end
            `STATE_SIDEINFO_PART2_3_LENGTH:
            begin
              $write("part2_3_length: %d\n", uut.HUFFMANDECODER.data[15:4]);
            end
            `STATE_SIDEINFO_BIG_VALUES:
            begin
              $write("big_values: %d -> x2 = %d\n", uut.HUFFMANDECODER.data[15:7], uut.HUFFMANDECODER.data[15:7]*2);
            end
            `STATE_SIDEINFO_GLOBAL_GAIN:
            begin
              $write("global_gain: %d\n",uut.HUFFMANDECODER.data[15:8]);
              $write("scalefac_compress: %d\n",uut.HUFFMANDECODER.data[7:4]);
            end
            `STATE_SIDEINFO_BLOCK_TYPE:
            begin
              $write("WSF: %d\n",uut.HUFFMANDECODER.data[15]);
              if (uut.HUFFMANDECODER.data[15] == 1)  // WSF == 1
              begin
                $write("block_type: %d\n",uut.HUFFMANDECODER.data[14:13]);
                $write("MBF: %d\n",uut.HUFFMANDECODER.data[12]);
              end
              else
              begin
                $write("block_type: 0\n");
                $write("MBF: 0\n");
              end
            end
            `STATE_SIDEINFO_TABLE_SELECT:
            begin
              $write("table_select");
              if (uut.HUFFMANDECODER.gpreg[15] == 1)  // WSF == 1
                $write("0,1: %d %d\n",uut.HUFFMANDECODER.data[15:11],uut.HUFFMANDECODER.data[10:6]);
              else
                $write("0,1,2: %d %d %d\n",uut.HUFFMANDECODER.data[15:11],uut.HUFFMANDECODER.data[10:6],uut.HUFFMANDECODER.data[5:1]);
            end
            `STATE_SIDEINFO_SUBBLOCK_GAIN:
            begin
              if (uut.HUFFMANDECODER.gpreg[15] == 1)
                $write("subblock_gain0,1,2: %d %d %d\n",uut.HUFFMANDECODER.data[15:13],uut.HUFFMANDECODER.data[12:10],uut.HUFFMANDECODER.data[9:7]);
            end
            `STATE_SIDEINFO_REGION_COUNT:
            begin
              if (uut.HUFFMANDECODER.gpreg[15] == 1)
              begin
                if (uut.HUFFMANDECODER.gpreg[2:0] == 3'b100)
                  $write("region0/1count: 8,12\n");
                else
                  $write("region0/1count: 7,13\n");
                $write("preflag, scalefac_scale, count1table_select: %d %d %d\n",uut.HUFFMANDECODER.data[6],uut.HUFFMANDECODER.data[5],uut.HUFFMANDECODER.data[4]);
              end
              else
              begin
              end
            end
            endcase
          end  // stage_sideinfo
          `STAGE_DECODE:
          begin
            case (uut.HUFFMANDECODER.state)
            `STATE_DECODE_SCALEFAC:
            begin
              case (uut.HUFFMANDECODER.step)
              `ifdef MISMATCH_BEFORE_PAUSE
              `else
              0:  mismatch = 0;
              `endif
              1:  // long - mixed
              begin
                `ifdef SHOW_SCALEFAC
                $write("long scalefac %d: %d\n",uut.HUFFMANDECODER.counter[4:0],uut.HUFFMANDECODER.data_slen1);
                `endif
              end
              2:  // short
              begin
                `ifdef SHOW_SCALEFAC
                case (uut.HUFFMANDECODER.counter[3:0])
                0,1,2,3,4,5:
                  $write("short scalefac %d: %d\n",uut.HUFFMANDECODER.counter[3:0],uut.HUFFMANDECODER.data_slen1);
                default:
                  $write("short scalefac %d: %d\n",uut.HUFFMANDECODER.counter[3:0],uut.HUFFMANDECODER.data_slen2);
                endcase
                `endif
              end
              3:  // long
              begin
                `ifdef SHOW_SCALEFAC
                case (uut.HUFFMANDECODER.counter[4:0])
                0,1,2,3,4,5:
                  if (uut.HUFFMANDECODER.gr == 0 || uut.HUFFMANDECODER.gpreg[3] == 0)  // scfsi[0]
                    $write("long scalefac %d: %d\n",uut.HUFFMANDECODER.counter[4:0],uut.HUFFMANDECODER.data_slen1);
                6,7,8,9,10:
                  if (uut.HUFFMANDECODER.gr == 0 || uut.HUFFMANDECODER.gpreg[2] == 0)  // scfsi[1]
                    $write("long scalefac %d: %d\n",uut.HUFFMANDECODER.counter[4:0],uut.HUFFMANDECODER.data_slen1);
                11,12,13,14,15:
                  if (uut.HUFFMANDECODER.gr == 0 || uut.HUFFMANDECODER.gpreg[1] == 0)  // scfsi[2]
                    $write("long scalefac %d: %d\n",uut.HUFFMANDECODER.counter[4:0],uut.HUFFMANDECODER.data_slen2);
                16,17,18,19,20:
                  if (uut.HUFFMANDECODER.gr == 0 || uut.HUFFMANDECODER.gpreg[0] == 0)  // scfsi[3]
                    $write("long scalefac %d: %d\n",uut.HUFFMANDECODER.counter[4:0],uut.HUFFMANDECODER.data_slen2);
                endcase
                `endif
              end  
              endcase
            end
            
            `STATE_DECODE_REQ_STEREO:
            begin
              case (uut.HUFFMANDECODER.step)
              0:
              begin
              `ifdef COMPARE_HUFFMAN
                decoded_sample = uut.HUFFMANDECODER.decoded_sample;
                if (uut.HUFFMANDECODER.decoded_sign)
                  decoded_sample = -decoded_sample;
                $fscanf(dfile,"%d",read_decoded_sample);
                `ifdef MISMATCH_TOLERANCE
                if ((decoded_sample - read_decoded_sample) < -`MISMATCH_TOLERANCE || (decoded_sample - read_decoded_sample) > `MISMATCH_TOLERANCE)
                `else
                if (decoded_sample != read_decoded_sample)
                `endif
                begin
                  mismatch = mismatch + 1;
                  $write("%d: *M* H-%d %d\n", uut.HUFFMANDECODER.counter,decoded_sample,read_decoded_sample);
                  //$stop(3);
                end
                //$write("%d: H-%d %d\n", uut.HUFFMANDECODER.counter,decoded_sample,read_decoded_sample);
                requantized_sample = uut.HUFFMANDECODER.req_result_signed;
                if (uut.HUFFMANDECODER.req_result_signed & 16'h8000)
                  requantized_sample = requantized_sample - 65536;
                $fscanf(rfile,"%d",read_requantized_sample);
                `ifdef MISMATCH_TOLERANCE
                if ((requantized_sample - read_requantized_sample) < -`MISMATCH_TOLERANCE || (requantized_sample - read_requantized_sample) > `MISMATCH_TOLERANCE)
                `else
                if (requantized_sample != read_requantized_sample)
                `endif
                begin
                  mismatch = mismatch + 1;
                  $write("%d: *M* Q-%d %d\n",uut.HUFFMANDECODER.counter,requantized_sample,read_requantized_sample);
                  $write("   H=%d ROM=%d is_adj=%d m1=%x m2=%x mo=%x expo=%d\n",
                    uut.HUFFMANDECODER.decoded_sample,
                    uut.HUFFMANDECODER.REQ_ROM_data_out,
                    uut.HUFFMANDECODER.is_adj,
                    uut.HUFFMANDECODER.mult_in1,
                    uut.HUFFMANDECODER.mult_in2,
                    uut.HUFFMANDECODER.mult_out,
                    uut.HUFFMANDECODER.exp_out
                  );
                  //$stop(3);
                end
                //$write("%d: Q-%d %d\n",uut.HUFFMANDECODER.counter,requantized_sample,read_requantized_sample);
              `endif
              end
              1:
              begin
              //$write("%d: msis = %d\n",uut.HUFFMANDECODER.counter,uut.HUFFMANDECODER.msis);
              `ifdef COMPARE_STEREO
                //stereo_sample = uut.HUFFMANDECODER.rightchannel_datain;
//		$write("R-%d %h\n", uut.HUFFMANDECODER.mac_address, uut.HUFFMANDECODER.mac_datain);
                stereo_sample = uut.HUFFMANDECODER.mac_datain;
                if (stereo_sample > 32767)
                  stereo_sample = stereo_sample - 65536;
                $fscanf(sfile,"%d",read_stereo_sample);
                `ifdef MISMATCH_TOLERANCE
                if ((stereo_sample - read_stereo_sample) < -`MISMATCH_TOLERANCE || (stereo_sample - read_stereo_sample) > `MISMATCH_TOLERANCE)
                `else
                if (stereo_sample != read_stereo_sample)
                `endif
                begin
                  mismatch = mismatch + 1;
                  $write("%d: *M* R-%d %d\n", uut.HUFFMANDECODER.counter, stereo_sample, read_stereo_sample);
                  //$stop(3);
                end
                `ifdef COMPARE_REORDER
                  //rightram[uut.HUFFMANDECODER.rightchannel_address] = stereo_sample;
                  rightram[uut.HUFFMANDECODER.mac_address] = stereo_sample;
                `endif
              `endif
              end
              2:
              begin
              `ifdef COMPARE_STEREO
                if ((uut.HUFFMANDECODER.js | uut.HUFFMANDECODER.stereo) && uut.HUFFMANDECODER.ch==1)
                begin
                  //stereo_sample = uut.HUFFMANDECODER.leftchannel_datain;
//		  $write("L-%d %h\n", uut.HUFFMANDECODER.mac_address, uut.HUFFMANDECODER.mac_datain);
                  stereo_sample = uut.HUFFMANDECODER.mac_datain;
                  if (stereo_sample > 32767)
                    stereo_sample = stereo_sample - 65536;
                  $fscanf(sfile,"%d",read_stereo_sample);
                  `ifdef MISMATCH_TOLERANCE
                  if ((stereo_sample - read_stereo_sample) < -`MISMATCH_TOLERANCE || (stereo_sample - read_stereo_sample) > `MISMATCH_TOLERANCE)
                  `else
                  if (stereo_sample != read_stereo_sample)
                  `endif
                  begin
                    mismatch = mismatch + 1;
                    $write("%d: *M* L-%d %d\n", uut.HUFFMANDECODER.counter, stereo_sample, read_stereo_sample);
                    //$stop(3);
                  end
                end
                `ifdef COMPARE_REORDER
                  //leftram[uut.HUFFMANDECODER.leftchannel_address] = stereo_sample;
                  leftram[uut.HUFFMANDECODER.mac_address] = stereo_sample;
                `endif
              `endif
              `ifdef SHOW_MISMATCH
                if (uut.HUFFMANDECODER.next_counter == 576)
                begin
                  $write("----------mismatch: %d\n", mismatch);
                  `ifdef MISMATCH_BEFORE_PAUSE
                  `else
                  if (mismatch > 0)
                    $stop(1);
                  `endif
                end
              `endif
              `ifdef COMPARE_REORDER
                if (uut.HUFFMANDECODER.next_counter == 576)
                begin
                  if (uut.HUFFMANDECODER.ch == 1)
                  begin
                    $write("----reordered samples:\n");
                    for (i=0;i<576;i=i+1)
                      $write("%d: L: %d R: %d\n",i,leftram[i],rightram[i]);
                  end
                end
              `endif
                if (uut.HUFFMANDECODER.counter > 576)
                begin
                  $write("counter over 576!!\n");
                  $stop(10);
                end
              end
              default:
              begin
              end
              endcase  // step
            end
            endcase  // state
          end  // stage DECODE
          `STAGE_POST_DECODE:
          begin
          end
          endcase  // stage
        end  // always
        
        
/*
        always @(uut.dpw_address)
        begin
          if (uut.dpw_address == 513)
          begin
            if (endsim == 1)
            begin
              $write("\n---------------END OF SIMULATION-----------------\n\n");
              $stop(2);
            end
            else
              endsim = 1;
          end
        end
*/        
        `ifdef MISMATCH_BEFORE_PAUSE
        always @(mismatch)
        begin
          if (mismatch >= `MISMATCH_BEFORE_PAUSE)
          begin
            $stop(3);
            mismatch = 0;
          end
        end
        `endif
	
// HUFFMAN Testbench end


endmodule
